#! /usr/local/bin/python

from getopt import getopt
from sys import argv, stdin, stderr, stdout, exit
from types import StringTypes, IntType

# Parse options
options, arguments = getopt(argv[1:], "mkv")
ms_format = False
known_mrca = False
verbose = False
def parse_positiveint(s):
  i = int(s)
  if i < 0:
    raise ValueError, "Negative value encountered"
  return i
    
for o in options:
  if o[0][1] == 'm':
    ms_format = True
  elif o[0][1] == 'k':
    known_mrca = True
  elif o[0][1] == 'v':
    verbose = True

# Read data
if arguments != []:
  infile = open(arguments[0], 'r')
else:
  infile = stdin
data = []
if ms_format:
  while True:
    s = infile.readline()
    if s[:2] == "//":
      for i in range(3):
        s = infile.readline()
      while s != "" and s[:2] != "//":
        s = s.strip()
        if s != "":
          data.append(s)
          if len(data[-1]) != len(data[0]):
            stderr.write("Data lines are not of the same length\n")
            exit(1)
        s = infile.readline()
    elif s != "":
      continue
    break
else:
  # getline(f): returns next non-empty, non-comment line from f, None if EOF
  def getline(f):
    while True:
      s = f.readline()
      if s == "":
        return None
      # Strip off trailing comment
      escaped = 0
      for i in range(0, len(s)):
        if s[i] == '#' and not escaped:
          s = s[:i]
          break
        if s[i] == '\\':
          escaped = not escaped
      # Strip off trailing whitespace
      s = s.rstrip()
      if s != "":
        break
    return s

  count = None
  while True:
    line = getline(infile)
    if line == None:
      break
    if line[0] == '>':
      # This line contains count of next sequence
      for i in xrange(1, len(line)):
        if line[i] != '>':
          count = parse_positiveint(line[i:])
        break
      else:
        # No count in line - reset
        count = None
      continue
    data.append((line, count))
    count = None
    if len(data[-1][0]) != len(data[0][0]):
      stderr.write("Data lines are not of the same length\n")
      exit(1)
  if arguments != []:
    infile.close()

# Add two counts
def add_counts(a, b):
  if a[0] == "" or b[0] == "":
    return (a[0] + b[0], a[1] + b[1])
  else:
    return (a[0] + "+" + b[0], a[1] + b[1])

# Output count to file
def output_count(a, fp, prefix = "", postfix = "\n"):
  fp.write(prefix);
  if a[0] == "":
    fp.write(str(a[1]))
  elif a[1] == 0:
    fp.write(a[0])
  else:
    fp.write(a[0] + "+" + str(a[1]))
  fp.write(postfix)

# Add initial count variable to each sequence, remove duplicates,
# and check for symbols not in {0, 1}
for i in xrange(len(data)):
  if data[i][1] == None:
    # Assign variable to sequences without count
    data[i] = (data[i][0], ("n" + str(i + 1), 0))
  else:
    data[i] = (data[i][0], ("", data[i][1]))
data.sort()
i = 0
while i < len(data):
  if i > 0 and data[i][0] == data[i - 1][0]:
    # Previous copy of this sequence
    data[i - 1] = (data[i - 1][0], add_counts(data[i - 1][1], data[i][1]))
    del data[i]
  else:
    for j in data[i][0]:
      if j not in "01":
        # Non-binary symbol encountered - delete sequence and issue warning
        stderr.write("WARNING: sequence '" + data[i][0] + "' contains non-binary symbols\n")
        del data[i]
        break
    else:
      # Kept this sequence - proceed with next one
      i += 1
if i == 0:
  # No sequences left in data set
  print "No sequences in data set; no ACs can be encountered"
  exit(0)
# Determine sequence length
n = len(data[0][0])

# Determine list of singletons
def handle_ac(ac):
  # Return a list of all reachable ACs
  reachable = []
  if known_mrca:
    # Only consider substituting a 1 with a 0
    subs = xrange(1, 2)
  else:
    # Consider substitutions going both ways
    subs = xrange(2)
  # Check each site for singleton
  for i in xrange(n):
    # Register occurrences of 0 and 1, respectively, for site i
    c = [-1, -1]
    for j in xrange(len(ac[0])):
      type = int(ac[0][j][i])
      if c[type] == -1:
        # First occurrence of this type in site i
        c[type] = j
      else:
        # Multiple occurrences of this type in site i
        c[type] = -2
    # Determine presence and effects of singletons
    for j in subs:
      if c[j] >= 0 and c[1 - j] != -1:
        # Singleton in site i
        news = ac[0][c[j]][:i] + str(1 - j) + ac[0][c[j]][i + 1:]
        newac = map(None, ac[0], ac[1])
        del newac[c[j]]
        for k in xrange(len(newac)):
          if news == newac[k][0]:
            # Mutation leads to sequence already present in AC
            newac[k] = (news, add_counts(newac[k][1], ("", 1)))
            break
        else:
          # Mutation leads to novel sequence
          newac.append((news, ("", 1)))
        newac.sort()
        newac = (tuple([k[0] for k in newac]), [k[1] for k in newac])
        reachable.append(newac)
  return reachable

# Starting from the ancestral configuration corresponding to the data set,
# find all types that we can reach
pending = {tuple([i[0] for i in data]): [i[1] for i in data]}
fixed = {}
while pending != {}:
  ac = pending.popitem()
  fixed[ac[0]] = ac[1]
  acs = handle_ac(ac)
  for ac in acs:
    if fixed.has_key(ac[0]):
      if fixed[ac[0]] != ac[1]:
        print "***", ac[0], fixed[ac[0]], ac[1]
    elif pending.has_key(ac[0]):
      if pending[ac[0]] != ac[1]:
        print "***", ac[0], pending[ac[0]], ac[1]
    else:
      pending[ac[0]] = ac[1]

expression = ("", 0)
while fixed != {}:
  ac = fixed.popitem()
  if verbose:
    print ac[0], ac[1]
  term = ("", 1)
  # Create term from the factors in for this AC
  for i in ac[1]:
    if i[0] == "":
      term = (term[0], term[1] * i[1])
    else:
      s = i[0]
      if i[1] != 0:
        s = "(" + s + "+" + str(i[1]) + ")"
      elif '+' in s:
        s = "(" + s + ")"
      term = (term[0] + s + "*", term[1])
  # Add term to overall expression
  if term[0] != "":
    if term[1] != 1:
      term = (str(term[1]) + "*" + term[0][:-1], 0)
    else:
      term = (term[0][:-1], 0)
  expression = add_counts(expression, term)

output_count(expression, stdout)
