#!/usr/bin/python
from sys import argv, stderr, stdout, exit
from copy import deepcopy

if len(argv) < 2:
  stderrr.write("No data file specified\n")
  exit(1)

try:
  historyfile = open(argv[2], 'w')
except:
  historyfile = stdout

# getline(f): returns next non-empty, non-comment line from f, None if EOF
def getline(f):
  while 1:
    s = f.readline()
    if s == "":
      return None
    # Strip off trailing comment
    escaped = 0
    for i in range(0, len(s)):
      if s[i] == '#' and not escaped:
        s = s[:i]
        break
      if s[i] == '\\':
        escaped = not escaped
    # Strip off trailing whitespace
    s = s.rstrip()
    if s != "":
      break
  return s

# Read data
infile = open(argv[1], 'r')
data = []
while 1:
  line = getline(infile)
  if line == None:
    break
  data.append(line)
  if len(data[-1]) != len(data[0]):
    stderr.write("Data lines are not of the same length\n")
    exit(1)

# Output data with numbering of columns and of sequences as specified
def output_data(data, sequences):
  format = "%" + `len(`max(sequences)`)` + "d: "
  stdout.write((len(`max(sequences)`) + 2) * " ")
  for i in range(1, len(data[0]) + 1):
    stdout.write(`i % 10`)
  stdout.write("\n")
  for i in range(0, len(data)):
    print (format + data[i]) % sequences[i]

# Input event
def input_event(data, sequence, history):
  choice = raw_input("(M)utation (C)oalescence (R)ecombination re(N)umbering (Q)uit (H)istory: ").lower()[0]
  if choice == 'm':
    # Mutation event - read site number
    try:
      column = int(raw_input("Site to apply mutation to: "))
    except:
      return
    # Check whether we can apply mutation
    if column < 1 or column > len(data[0]):
      stderr.write("The data doesn't contain a site " + `column` + "\n")
      return
    column -= 1
    s0 = -1
    s1 = -1
    for i in range(0, len(data)):
      if data[i][column] == '0':
        if s0 == -1:
          s0 = i
        else:
          s0 = -2
      elif data[i][column] == '1':
        if s1 == -1:
          s1 = i
        else:
          s1 = -2
    if s1 >= 0:
      # We can mutate a 1 to a 0
      data[s1] = data[s1][:column] + '0' + data[s1][column + 1:]
      history.append("Mutated 1 in site " + `column + 1` + ", sequence " + `sequence[s1]` + " to a 0")
    elif s0 >= 0:
      # We can mutate a 0 to a 1
      data[s0] = data[s0][:column] + '1' + data[s0][column + 1:]
      history.append("Mutated 0 in site " + `column + 1` + ", sequence " + `sequence[s1]` + " to a 1")
    else:
      stderr.write("A mutation cannot be applied to column " + `column + 1` + "\n")
  elif choice == 'c':
    # Coalescence event - read sequence numbers
    try:
      s1 = int(raw_input("First sequence in coalescence:  "))
    except:
      return
    try:
      s1 = sequence.index(s1)
    except ValueError:
      stderr.write("There is no sequence " + `s1` + " in data\n")
      return
    try:
      s2 = int(raw_input("Second sequence in coalescence: "))
    except:
      return
    try:
      s2 = sequence.index(s2)
    except ValueError:
      stderr.write("There is no sequence " + `s2` + " in data\n")
      return
    for i in range(0, len(data[s1])):
      if data[s1][i] == '0' and data[s2][i] == '1' or data[s1][i] == '1' and data[s2][i] == '0':
        stderr.write("Position " + `i + 1` + " is incompatible for sequences " + `sequence[s1]` + " and " + `sequence[s2]` + "\n")
        return
    # Perform coalescence
    history.append("Coalesced sequences " + `sequence[s1]` + " and " + `sequence[s2]`)
    for i in range(0, len(data[s1])):
      if data[s1][i] not in "01":
        data[s1] = data[s1][:i] + data[s2][i] + data[s1][i + 1:]
    del data[s2]
    del sequence[s2]
  elif choice == 'r':
    # Recombination event - read sequence number and site number
    try:
      s = int(raw_input("Sequence to apply recombination to: "))
    except:
      return
    try:
      s = sequence.index(s)
    except ValueError:
      stderr.write("There is no sequence " + `s` + " in data\n")
      return
    try:
      column = int(raw_input("Site the recombination happens after: "))
    except:
      return
    # Check whether we can apply recombination
    if column < 1 or column > len(data[s]):
      stderr.write("The data doesn't contain a site " + `column` + "\n")
      return
    data.append(column * "x" + data[s][column:])
    sequence.append(max(sequence) + 1)
    data[s] = data[s][:column] + (len(data[s]) - column) * "x"
    history.append("Split sequence " + `sequence[s]` + " after site " + `column` + "; the postfix became new sequence " + `sequence[-1]`)
  elif choice == 'n':
    # Renumber a sequence - read old and new sequence number
    try:
      s = int(raw_input("Sequence to renumber: "))
    except:
      return
    try:
      s = sequence.index(s)
    except ValueError:
      stderr.write("There is no sequence " + `s` + " in data\n")
      return
    try:
      news = int(raw_input("New sequence number: "))
    except:
      return
    if news in sequence:
      stderr.write("There is already one sequence " + `news` + " in the data\n")
      return
    history.append("Renamed sequence " + `sequence[s]` + " to " + `news`)
    sequence[s] = news
  elif choice == 'q':
    if historyfile != stdout:
      historyfile.close()
    exit(0)
  elif choice == 'h':
    map(historyfile.write, map(lambda x: x + "\n", history))
  else:
    stderr.write("`" + choice + "' is not a valid choice\n")

sequences = range(1, len(data) + 1)
history = ["Start of history"]
while 1:
  output_data(data, sequences)
  input_event(data, sequences, history)
