/*********************************************************************

    expression.h
  
    Function for parsing a list of expressions into a program. These are
    only intended for use by the associated parser generated from the
    expression.y file. The expression_parse function declared in
    expression.h provides the standard interface for parsing a string.
		
    Rune Lyngsoe (lyngsoe@stats.ox.ac.uk), June 2007

**********************************************************************/

#ifndef _EXPRESSION_BISON_H
#define _EXPRESSION_BISON_H

#include "expression.h"
#include "llist.h"

Expression *expression_parseor(Program *p, Expression *a, Expression *b);
Expression *expression_parseand(Program *p, Expression *a, Expression *b);
Expression *expression_parsexor(Program *p, Expression *a, Expression *b);
Expression *expression_parsenot(Program *p, Expression *a);
Expression *expression_parseless(Program *p, Expression *a, Expression *b);
Expression *expression_parsegreater(Program *p, Expression *a, Expression *b);
Expression *expression_parselessequal(Program *p, Expression *a,
				      Expression *b);
Expression *expression_parsegreaterequal(Program *p, Expression *a,
					 Expression *b);
Expression *expression_parseequal(Program *p, Expression *a, Expression *b);
Expression *expression_parsenotequal(Program *p, Expression *a, Expression *b);
Expression *expression_parseadd(Program *p, Expression *a, Expression *b);
Expression *expression_parsesub(Program *p, Expression *a, Expression *b);
Expression *expression_parsemul(Program *p, Expression *a, Expression *b);
Expression *expression_parsediv(Program *p, Expression *a, Expression *b);
Expression *expression_parseneg(Program *p, Expression *a);
Expression *expression_parseexp(Program *p, Expression *a, Expression *b);
Program *expression_parseprogram(Program *p, Expression *e);
Program *expression_parsedefinitions(Program *p, Program *e);
Program *expression_parsevariabledefinition(Program *p, char *name,
					    Expression *e);
Program *expression_parsefunctiondefinitionhead(Program *p, LList *newnames);
Program *expression_parsefunctiondefinition(Program *p, Expression *e);
LList *expression_parsedeclaration(char *name, LList *locals);
LList *expression_parsefirstparameter(char *name);
LList *expression_parseparameters(LList *p, char *name);
Expression *expression_parsenumber(double x);
Expression *expression_parsevariable(Program *p, char *name);
Expression *expression_parsechoice(Expression *t, Expression *c,
				   Expression *a);
Expression *expression_parseapplication(Program *p, char *name, LList *a);
LList *expression_parsefirstargument(Expression *e);
LList *expression_parsearguments(LList *a, Expression *e);
extern Program *expression_currentprogram;

#endif
