/* expression.flex: Decription of tokens of kwarg expressions.
 *
 * Rune Lyngsoe, March 2007
 */
%{
#include "expression_grammar.tab.h"
#include <stdio.h>
#include <stdlib.h>

static void _unrecognised(char *);
extern int expression_definitioncount;
extern int expression_currentdefinition;
%}

DIGITS     [[:digit:]]+
INTEGER    [1-9]{DIGITS}?|"0"
FLOAT      {INTEGER}?(("."{DIGITS}(("e"|"E")("+"|"-")?{DIGITS})?)|(("e"|"E")("+"|"-")?{DIGITS}))
NAME       [[:alpha:]][[:alnum:]]*

%option noyywrap
%s RHS

%%
	if (expression_definitioncount == 0) BEGIN(RHS);
{INTEGER}       |
{FLOAT}         yylval.number = atof(yytext); return NUMBER;
<INITIAL>{NAME} BEGIN(RHS); yylval.identifier = strdup(yytext); return DEFNAME;
<RHS>{NAME}     yylval.identifier = strdup(yytext); return IDENTIFIER;
"+"             return ADD;
"-"             return *yytext;
"*"             return MUL;
"/"             return DIV;
"^"|"**"        return EXP;
"<"             return LESS;
">"             return GREATER;
"\\"|"\\\\"     return XOR;
"&"|"&&"        return AND;
"|"|"||"        return OR;
"!"|"¬"         return NOT;
"?"             return CHOOSE;
":"             return ALTERNATIVE;
"<="|"=<"       return LESSEQUAL;
">="|"=>"       return GREATEREQUAL;
"=="            return EQUAL;
"!="|"<>"|"><"  return NOTEQUAL;
"("|")"|","|"=" return *yytext;
";"             if (++expression_currentdefinition < expression_definitioncount) BEGIN(INITIAL); return *yytext;
[ \t\n]
.               _unrecognised(yytext);
%%

static void _unrecognised(char *symbol)
{
  fprintf(stderr, "Unrecognised symbol %s in expression\n", symbol);
  exit(1);
}
