/*********************************************************************

    expression.h
  
    Data structures and functions for storing and resolving arithmetic
    expressions
		
    Rune Lyngsoe (lyngsoe@stats.ox.ac.uk), June 2007

**********************************************************************/

#ifndef _EXPRESSION_H
#define _EXPRESSION_H

#include <stdarg.h>
#include "elist.h"
#include "llist.h"

/* Datatypes */
typedef enum { EXPRESSION_NUMBER, EXPRESSION_BOOLEAN, EXPRESSION_VARIABLE,
	       EXPRESSION_BUILTIN, EXPRESSION_CONSTANT, EXPRESSION_DECLARATION,
	       EXPRESSION_APPLICATION, EXPRESSION_CHOICE, EXPRESSION_ERROR
} ExpressionType;

typedef struct _Expression {
  ExpressionType type;
  union {
    double number;
    int boolean;
    int variable;
    struct {
      struct _Expression *(*f)(struct _Expression **);
      int arity;
    } builtin;
    double *constant;
    struct {
      struct _Expression *body;
      int arity;
    } declaration;
    struct {
      int f;
      int arity;
      struct _Expression **arguments;
    } application;
    struct {
      struct _Expression *test;
      struct _Expression *consequence;
      struct _Expression *alternate;
    } choice;
    char *error;
  } content;
} Expression;

typedef struct _Declaration {
  char *name;
} Declaration;

typedef struct _Program {
  EList *program;
  EList *builtin;
  EList *namespace;
} Program;

void expression_buildinfunction(Program *p, Expression *(*f)(Expression **),
				int arity, char *name);
void expression_buildinfunctions(Program *p, void *f[][3]);
void expression_buildinvariable(Program *p, double *value, char *name);
void expression_buildinvariables(Program *p, void *f[][2]);
Program *expression_initialise();
void expression_destroy(Program *p,
			void (*free_function)
			(Expression *(*)(Expression **)),
			void (*free_variable)(double *),
			void (*free_name)(char *));
void expression_parse(Program *prg, char *statements);
LList *expression_parse_and_check(Program *prg, char *statements);
double expression_interpret(Program *prg);

#endif
