#! /usr/local/bin/python
from sys import argv, exit, stdin, stdout
from os import tmpnam, popen
from getopt import getopt
from operator import add
from copy import deepcopy
from string import digits
from pickle import dump, load

def print_usage(s):
  print s, "[Options]"
  print "Legal options are:"
  print " -nN Each data set consists of N sequences (default is 5)"
  print " -lN Data sets contain N sites (default is 2)"
  print " -cS S is the name of the program invoked to find R_min (default beagle)"
  print " -dS S is the name of the program invoked to approximate R_min (default myers)"
  print " -rS Recovery information is saved to file S before handling each data set"
  print " -s  Silent (do not output each data set with AC count)"
  print " -b  Run the enumeration of data sets backwards"
  print " -?, -h, -H Print this information and stop"
  exit(0)

# Parse options
options, args = getopt(argv[1:], "n:l:c:d:r:sb?hH")
n = 5
l = 2
command = "beagle"
command2 = "myers"
recovery_file = None
silent = False
little_endian = True
for o in options:
  if o[0][1] == 'n':
    n = int(o[1])
  elif o[0][1] == 'l':
    l = int(o[1])
  elif o[0][1] == 'c':
    command = o[1]
  elif o[0][1] == 'd':
    command2 = o[1]
  elif o[0][1] == 'r':
    recovery_file = o[1]
  elif o[0][1] == 's':
    silent = True
  elif o[0][1] == 'b':
    little_endian = False
  elif o[0][1] in "?hH":
    print_usage(argv[0])

if args != []:
  f = open(args[0])
  n, l, rD, max_rec, max_recD, max_app, max_appD, max_diff, max_diffD, sum, count = load(f)
  f.close()
else:
  # Create initial data set
  max_rec = -1
  max_recD = None
  max_app = -1
  max_appD = None
  max_diff = -1
  max_diffD = None
  sum = 0
  count = 0
  # No recovered data set
  rD = None
filename = tmpnam()

# Find integer number in string s
def parse_output(s):
  i = 0
  while i < len(s) and s[i] not in digits:
    i += 1
  if i >= len(s):
    stderr.write(s + "\n")
    return None
  j = i + 1
  while j < len(s) and s[j] in digits:
    j += 1
  return int(s[i:j])

# Print a data set
def output_data(D):
  if D == None or len(D) == 0:
    print "{}",
  elif len(D) == 1:
    print "{" + reduce(add, D[0], "") + "}",
  else:
    print "{" + reduce(add, D[0], "") + ",",
    for i in D[1:-1]:
      print reduce(add, i , "") + ",",
    print reduce(add, D[-1], "") + "}",

def build_state(D, i):
  global max_rec
  global max_recD
  global max_app
  global max_appD
  global max_diff
  global max_diffD
  global sum
  global count
  global rD
  if i == n:
    # We have built a full data set
    if rD != None:
      # We have restored recovery data set
      rD = None
      return
    if not silent:
      output_data(D)
    f = open(filename, "w")
    for s in D:
      f.write(reduce(add, s, "") + "\n")
    f.close()
    # Run beagle on data set
    f = popen(command + " " + filename)
    s = f.readline()
    while s != "":
      t = s.strip()
      s = f.readline()
    f.close()
    Rmin = parse_output(t)
    if Rmin > max_rec:
      max_rec = Rmin
      max_recD = [deepcopy(D)]
    elif Rmin == max_rec:
      max_recD.append(deepcopy(D))
    # Run myers on data set
    f = popen(command2 + " " + filename)
    s = f.readline()
    while s != "":
      t = s.strip()
      s = f.readline()
    f.close()
    Rh = parse_output(t)
    if Rh > max_app:
      max_app = Rh
      max_appD = [deepcopy(D)]
    elif Rh == max_app:
      max_appD.append(deepcopy(D))
    if Rmin - Rh > max_diff:
      max_diff = Rmin - Rh
      max_diffD = [deepcopy(D)]
    elif Rmin - Rh == max_diff:
      max_diffD.append(deepcopy(D))
    if not silent:
      print Rmin, Rh
      stdout.flush()
    sum += Rmin
    count += 1
    if recovery_file != None:
      # Save recovery information
      f = open(recovery_file, 'w')
      dump((n, l, D, max_rec, max_recD, max_app, max_appD, max_diff, max_diffD, sum, count), f)
      f.close()
  else:
    # Add an extra sequence
    if little_endian:
      # Increment sequence s
      def increment(s, D):
        try:
          i = s.index('0')
        except ValueError:
          return None
        return i * ['0'] + ['1'] + s[i + 1:]
      if D == []:
        sequence = l * ['0']
    else:
      # Decrement sequence s
      def increment(s, D):
        for i in range(len(s)):
          if s[i] != D[-1][i]:
            break
        else:
          return None
        i = s.index('1')
        try:
          j = i + c[i:].index('0')
        except ValueError:
          return None
        return i * ['1'] + ['0'] + s[i + 1:]
      if D == []:
        sequence = l * ['1']
    if rD != None and len(rD) > len(D):
      # Recover old data set
      sequence = rD[len(D)]
    elif D != []:
      sequence = increment(D[-1], D)
    while sequence != None:
      build_state(D + [sequence], i + 1)
      sequence = increment(sequence, D)

build_state([], 0)
print "---"
for D in max_recD:
  output_data(D)
print max_rec,
if count != 0:
  print "%.2f" % (sum / float(count))
else:
  print
for D in max_appD:
  output_data(D)
print max_app
for D in max_diffD:
  output_data(D)
print max_diff
popen("rm " + filename).close()
