#! /usr/local/bin/python
from sys import argv, exit, stdin, stdout
from os import tmpnam
from getopt import getopt
from os import popen
from copy import deepcopy
from string import digits
from pickle import dump, load

def print_usage(s):
  print s, "[Options]"
  print "Legal options are:"
  print " -0N Each site should contain N 0s (default is 3)"
  print " -1N Each site should contain N 1s (default is 2)"
  print " -lN Data sets contain N sites (default is 2)"
  print " -cS S is the name of the program invoked to count ACs (default is greven)"
  print " -dS S is the name of the program invoked to compute R_min (default is beagle)"
  print " -rS Recovery information is saved to file S before handling each data set"
  print " -s  Silent (do not output each data set with AC count)"
  print " -b  Run the enumeration of data sets backwards"
  print " -?, -h, -H Print this information and stop"
  exit(0)

# Parse options
options, args = getopt(argv[1:], "0:1:l:c:d:r:sb?hH")
n0 = 3
n1 = 2
l = 2
command = "greven"
command2 = "beagle"
recovery_file = None
silent = False
little_endian = True
for o in options:
  if o[0][1] == '0':
    n0 = int(o[1])
  elif o[0][1] == '1':
    n1 = int(o[1])
  elif o[0][1] == 'l':
    l = int(o[1])
  elif o[0][1] == 'c':
    command = o[1]
  elif o[0][1] == 'd':
    command2 = o[1]
  elif o[0][1] == 'r':
    recovery_file = o[1]
  elif o[0][1] == 's':
    silent = True
  elif o[0][1] == 'b':
    little_endian = False
  elif o[0][1] in "?hH":
    print_usage(argv[0])

if args != []:
  f = open(args[0])
  n0, n1, l, rD, max_states, max_D, sum, count = load(f)
  f.close()
  n = n0 + n1
else:
  # Create initial data set
  n = n0 + n1
  max_states = -1
  sum = 0
  count = 0
  max_D = None
  # No recovered data set
  rD = None
D = [""] * n
filename = tmpnam()

# Find integer number in string s
def parse_output(s):
  i = 0
  while i < len(s) and s[i] not in digits:
    i += 1
  if i >= len(s):
    stderr.write(s + "\n")
    return None
  j = i + 1
  while j < len(s) and s[j] in digits:
    j += 1
  return int(s[i:j])

# Print a data set
def output_data(D):
  if len(D) == 0:
    print "{}",
  elif len(D) == 1:
    print "{" + D[0] + "}",
  else:
    print "{" + D[0] + ",",
    for i in D[1:-1]:
      print i + ",",
    print D[-1] + "}",

def build_state(D, i):
  global max_states
  global max_D
  global sum
  global count
  global rD
  if i == l:
    # We have built a full data set
    if rD != None:
      # We have restored recovery data set
      rD = None
      return
    if not silent:
      output_data(D)
    f = open(filename, "w")
    for s in D:
      f.write(s + "\n")
    f.close()
    # Run greven on data set
    f = popen(command + " " + filename)
    s = f.readline()
    while s != "":
      t = s.strip()
      s = f.readline()
    states = parse_output(t)
    if not silent:
      print states
      stdout.flush()
    sum += states
    count += 1
    if states > max_states:
      max_states = states
      max_D = deepcopy(D)
    if recovery_file != None:
      # Save recovery information
      f = open(recovery_file, 'w')
      dump((n0, n1, l, D, max_states, max_D, sum, count), f)
      f.close()
  else:
    # Add an extra column
    if little_endian:
      # Increment column c, making sure that adding it to D will still leave
      # D lexicographically sorted
      def increment(c, D):
        i = c.index('0')
        try:
          j = i + c[i:].index('1')
        except ValueError:
          return None
        c = (j - i - 1) * ['0'] + (i + 1) * ['1'] + ['0'] + c[j + 1:]
        # Check consistency with D
        for i in range(n - 1):
          if D[i] == D[i + 1] and c[i] == '1' and c[i + 1] == '0':
            return increment(c, D)
        return c
      column = n0 * ['0'] + n1 * ['1']
    else:
      # Decrement column c, making sure that adding it to D will still leave
      # D lexicographically sorted
      def increment(c, D):
        i = c.index('1')
        try:
          j = i + c[i:].index('0')
        except ValueError:
          return None
        c = (j - i - 1) * ['1'] + (i + 1) * ['0'] + ['1'] + c[j + 1:]
        # Check consistency with D
        for i in range(n - 1):
          if D[i] == D[i + 1] and c[i] == '1' and c[i + 1] == '0':
            return increment(c, D)
        return c
      column = n1 * ['1'] + n0 * ['0']
      if D[n1 - 1] == D[n1]:
        column = increment(column, D)
    if rD != None:
      # Recover old data set
      if len(rD[0]) > len(D[0]):
        column = map(lambda x: x[len(D[0])], rD)
      else:
        if rcolumn != None:
          column = rcolumn
        rD = None
    while column != None:
      build_state(map(lambda x, y: x + y, D, column), i + 1)
      column = increment(column, D)

build_state(D, len(D[0]))
f = open(filename, "w")
for s in max_D:
  f.write(s + "\n")
f.close()
f = popen(command2 + " " + filename)
s = f.readline()
while s != "":
  t = s.strip()
  s = f.readline()
recombinations = parse_output(t)

output_data(max_D)
print max_states, recombinations, "%.2f" % (sum / float(count))
popen("rm " + filename).close()
